// originates from Ruetsch/Oster: Getting Started with CUDA
#include <assert.h>
#include <cutil_inline.h>

__global__ void inc_gpu(double *a, int N)
{
  int idx = blockIdx.x * blockDim.x + threadIdx.x;
  if (idx < N)
      a[idx] = a[idx] + 1;
}



int main(void)
{
 double *a_h, *b_h; // host data
 double *a_d, *b_d; // device data
 int N = 1400, nBytes, i ;
 nBytes = N*sizeof(double);

 int blocksize=32; 
 dim3 dimBlock (blocksize);
 dim3 dimGrid( ceil( N / (double)blocksize) );


 a_h = (double *)malloc(nBytes);
 b_h = (double *)malloc(nBytes);
 cudaMalloc((void **) &a_d, nBytes);
 cudaMalloc((void **) &b_d, nBytes);

 for (i=0; i<N; i++) a_h[i] = 100.f + i;

 cutilSafeCall(cudaMemcpy(a_d, a_h, nBytes, cudaMemcpyHostToDevice));
 cudaMemcpy(b_d, a_d, nBytes, cudaMemcpyDeviceToDevice);
  
 inc_gpu<<<dimGrid, dimBlock>>>(b_d, N);

 cudaMemcpy(b_h, b_d, nBytes, cudaMemcpyDeviceToHost);

 for (i=0; i< N; i++) assert( a_h[i] == b_h[i]-1.0 );

 free(a_h); free(b_h); cudaFree(a_d); cudaFree(b_d);
 return 0;
}