// originates from Ruetsch/Oster: Getting Started with CUDA
#include <cassert>
#include <iostream>
using namespace std;
// #include <cutil_inline.h>

//    kernel function on device
__global__ void inc_gpu(double *a, int N)
{
  int idx = blockIdx.x * blockDim.x + threadIdx.x;
  if (idx < N)
      a[idx] = a[idx] + 1;
}


//  main function on host
int main(void)
{
 double *a_h, *b_h; // host data
 double *a_d, *b_d; // device data
 int N = 140000, nBytes, i ;
 nBytes = N*sizeof(double);

 int blocksize=512;
 dim3 dimBlock (blocksize);
 dim3 dimGrid( ceil( N / static_cast<double>(blocksize)) );


 a_h = new double [N];
 b_h = new double [N];
// allocate memory on device
 cudaMalloc((void **) &a_d, nBytes);
 cudaMalloc((void **) &b_d, nBytes);

 for (i=0; i<N; i++) a_h[i] = 100.f + i;

// copy data:  host --> device
//  cutilSafeCall(cudaMemcpy(a_d, a_h, nBytes, cudaMemcpyHostToDevice));
cudaMemcpy(a_d, a_h, nBytes, cudaMemcpyHostToDevice);
cudaMemcpy(b_d, a_d, nBytes, cudaMemcpyDeviceToDevice);

// call the kernel function with  dimGrid.x * dimBlock.x threads
 inc_gpu<<<dimGrid, dimBlock>>>(b_d, N);

// copy data:  device --> host
 cudaMemcpy(b_h, b_d, nBytes, cudaMemcpyDeviceToHost);

 for (i=0; i< N; i++) assert( a_h[i] == b_h[i]-1.0 );

 delete [] a_h; delete [] b_h; cudaFree(a_d); cudaFree(b_d);
 return 0;
}
